"""
Containers Module - Checks for container-related metrics.
Used for detecting differences in containers across active/standby hosts.
"""
from ..base import BaseExporter
from ..utils import run_command_silent, parse_json_silent, debug_log


class CustomExporter(BaseExporter):
    """
    Exporter for container-related checks.
    Runs checks related to container differences and reports metrics.
    """
    def metric_config(self):
        """Define metrics for container-related checks"""
        return {
            "dp_hactl_missing_containers_total": {"type": "Gauge", "labels": ["type", "problem"]},
        }

    def generate(self):
        """Run all container-related checks and collect metrics"""
        debug_log("Running container diff checks")
        # The utility functions will handle errors silently
        self._check_container_diff()

    def _check_container_diff(self):
        """Check for container differences between hosts"""
        missing_standby_metric = self.metrics.get("dp_hactl_missing_containers_total")
        
        debug_log("Running dp-ha-ctl container-diff command")
        # Use the silent command utility that handles errors
        result = run_command_silent(
            ["dp-ha-ctl", "container-diff"],
            timeout=15
        )
            
        # If we get here, the command executed successfully
        
        # Check if the output is empty
        if not result.stdout.strip():
            debug_log("No missing containers found (empty output)")
            # No output means no missing containers, set counts to 0
            missing_standby_count = 0
            missing_primary_count = 0
            # Always use all defined labels, with "none" for problem in normal cases
            missing_standby_metric.labels(type="missing_standby", problem="none").set(missing_standby_count)
            missing_standby_metric.labels(type="missing_primary", problem="none").set(missing_primary_count)
        else:
            debug_log("Processing container diff output")
            # Parse JSON with the silent utility
            container_diff_output = parse_json_silent(result.stdout)
            
            # Extract the counts, default to 0 if missing or empty
            missing_standby_count = len(container_diff_output.get("ContainerDiff", {}).get("missing_standby", []))
            missing_primary_count = len(container_diff_output.get("ContainerDiff", {}).get("missing_primary", []))
            
            debug_log(f"Found {missing_standby_count} missing standby and {missing_primary_count} missing primary containers")
            
            # Always use all defined labels, with "none" for problem in normal cases
            missing_standby_metric.labels(type="missing_standby", problem="none").set(missing_standby_count)
            missing_standby_metric.labels(type="missing_primary", problem="none").set(missing_primary_count)
