"""
Logging Checks Module - Checks for log-related metrics.
Used for metrics relating to log entries and patterns.
"""
import os
import datetime
from ..base import BaseExporter
from ..utils import debug_log


class CustomExporter(BaseExporter):
    """
    Exporter for logging-related checks.
    Runs checks related to log files and reports metrics.
    """
    def metric_config(self):
        """Define metrics for logging-related checks"""
        return {
            "hamanager_redis_auth_failures": {"type": "Gauge", "labels": []},
            "hamanager_redis_timeouts": {"type": "Gauge", "labels": []},
        }

    def generate(self):
        """Run all logging-related checks and collect metrics"""
        debug_log("Running logging-related checks")
        # The utility functions will handle errors silently
        self._check_redis_auth_failures()
        self._check_redis_timeouts()

    def _count_log_occurrences(self, search_pattern, time_window):
        """Count occurrences in logs within the last N minutes"""
        log_dir = "/var/log/hamanager/"
        
        debug_log(f"Counting occurrences of '{search_pattern}' in logs from last {time_window} minutes")
        
        # Get the timestamp N minutes ago
        start_time = datetime.datetime.now() - datetime.timedelta(minutes=time_window)
        
        # Initialize counter
        occurrences = 0
        
        # We only need to check the current log file since we're only looking at the last 5 minutes
        log_file = "hamanager.log"
        full_path = os.path.join(log_dir, log_file)
        debug_log(f"Checking current log file: {full_path}")
        
        try:
            # We use a regular try/except here because we want to continue rather than exit
            # if the file can't be read
            with open(full_path, 'r') as f:
                for line in f:
                    # Extract first 19 chars of timestamp (YYYY-MM-DD HH:MM:SS) from log line
                    try:
                        timestamp_str = line[:19]
                        timestamp = datetime.datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S")
                        if timestamp >= start_time and search_pattern in line:
                            occurrences += 1
                    except (ValueError, IndexError):
                        continue
        except Exception as e:
            debug_log(f"Error reading file {log_file}: {str(e)}")
        
        debug_log(f"Found {occurrences} occurrences of the pattern")
        return occurrences

    def _check_redis_auth_failures(self, time_window=5):
        """Check for Redis auth failures in logs"""
        auth_failures_metric = self.metrics.get("hamanager_redis_auth_failures")
        
        search_pattern = "invalid username-password pair"
        auth_failures = self._count_log_occurrences(search_pattern, time_window)
        
        auth_failures_metric.set(auth_failures)

    def _check_redis_timeouts(self, time_window=5):
        """Check for Redis timeouts in logs"""
        timeouts_metric = self.metrics.get("hamanager_redis_timeouts")
        
        search_pattern = "Redis exception:.*Timeout reading from socket"
        timeouts = self._count_log_occurrences(search_pattern, time_window)
        
        timeouts_metric.set(timeouts)
