"""
Systemctl Module - Checks for systemctl-based services.
Used for metrics relating to system service status.
"""
from ..base import BaseExporter
from ..utils import debug_log, run_systemctl_command


class CustomExporter(BaseExporter):
    """
    Exporter for systemctl-based checks.
    Runs checks related to system services and reports metrics.
    """
    def metric_config(self):
        """Define metrics for systemctl-based checks"""
        return {
            "podman_socket_active": {"type": "Gauge", "labels": []},
            "hamanager_service_active": {"type": "Gauge", "labels": []},
        }

    def generate(self):
        """Run all systemctl-based checks and collect metrics"""
        debug_log("Running systemctl-based checks")
        # The utility functions will handle errors silently
        self._check_podman_socket()
        self._check_hamanager_service()

    def _check_podman_socket(self):
        """Check if podman.socket is active"""
        podman_socket_metric = self.metrics.get("podman_socket_active")
        
        debug_log("Checking if podman.socket is active")
        # Use the special systemctl command utility
        is_active = run_systemctl_command("podman.socket", timeout=5)
        
        # Convert boolean to 1/0 for metric
        podman_socket_active = 1 if is_active else 0
        debug_log(f"podman.socket active: {is_active}")
        podman_socket_metric.set(podman_socket_active)

    def _check_hamanager_service(self):
        """Check if hamanager.service is active"""
        hamanager_service_metric = self.metrics.get("hamanager_service_active")
        
        debug_log("Checking if hamanager.service is active")
        # Use the special systemctl command utility
        is_active = run_systemctl_command("hamanager.service", timeout=5)
        
        # Convert boolean to 1/0 for metric
        hamanager_service_active = 1 if is_active else 0
        debug_log(f"hamanager.service active: {is_active}")
        hamanager_service_metric.set(hamanager_service_active)
